@@ -19,6 +19,7 @@ from datetime import datetime, date
 from typing import Union, List, Type
 
 import firefly.domain as ffd
+import regex
 
 
 class EntityAttributeSpy:
@@ -78,6 +79,14 @@ class Attr:
     def startswith(self, value):
         return BinaryOp(self.attr, 'startswith', value)
 
+    def lower(self):
+        self.attr = AttributeString(f'LOWER({self.attr})')
+        return self
+
+    def upper(self):
+        self.attr = AttributeString(f'UPPER({self.attr})')
+        return self
+
     def endswith(self, value):
         return BinaryOp(self.attr, 'endswith', value)
 
@@ -161,7 +170,7 @@ class BinaryOp:
 
     def matches(self, data: Union[ffd.Entity, dict]) -> bool:
         if isinstance(data, ffd.Entity):
-            data = data.to_dict()
+            data = data.to_dict(force_all=True)
 
         return self._do_match(self, data)
 
@@ -169,18 +178,30 @@ class BinaryOp:
         if isinstance(bop.lhv, BinaryOp):
             lhv = self._do_match(bop.lhv, data)
         elif isinstance(bop.lhv, AttributeString):
-            lhv = data[bop.lhv]
+            if '(' in bop.lhv:
+                lhv = self._parse_attribute_string(bop.lhv, data)
+            else:
+                lhv = data[bop.lhv]
         elif isinstance(bop.lhv, Attr):
-            lhv = data[bop.lhv.attr]
+            if '(' in bop.lhv.attr:
+                lhv = self._parse_attribute_string(bop.lhv.attr, data)
+            else:
+                lhv = data[bop.lhv.attr]
         else:
             lhv = bop.lhv
 
         if isinstance(bop.rhv, BinaryOp):
             rhv = self._do_match(bop.rhv, data)
         elif isinstance(bop.rhv, AttributeString):
-            rhv = data[bop.rhv]
+            if '(' in bop.rhv:
+                rhv = self._parse_attribute_string(bop.rhv, data)
+            else:
+                rhv = data[bop.rhv]
         elif isinstance(bop.rhv, Attr):
-            rhv = data[bop.rhv.attr]
+            if '(' in bop.rhv.attr:
+                rhv = self._parse_attribute_string(bop.rhv.attr, data)
+            else:
+                rhv = data[bop.rhv.attr]
         else:
             rhv = bop.rhv
 
@@ -213,6 +234,24 @@ class BinaryOp:
 
         raise ffd.LogicError(f"Don't know how to handle op: {bop.op}")
 
+    @staticmethod
+    def _remove_function_calls(attr: str):
+        return list(map(lambda rr: rr[2], regex.findall(r'((\w)\((?R)\))|(\w+)', attr))).pop()
+
+    @staticmethod
+    def _parse_attribute_string(attr: str, data: dict):
+        matches = list(map(lambda rr: rr[2], regex.findall(r'((\w)\((?R)\))|(\w+)', attr)))
+        attribute = matches.pop()
+        value = data[attribute]
+
+        for func in reversed(matches):
+            if func == 'LOWER':
+                value = value.lower()
+            elif func == 'UPPER':
+                value = value.upper()
+
+        return value
+
     def __and__(self, other):
         return BinaryOp(self, 'and', other)
 
@@ -233,9 +272,12 @@ class BinaryOp:
                     data['l'] = {'l': 1, 'o': '==', 'r': 1}
                 else:
                     data['l'] = {'l': 1, 'o': '!=', 'r': 1}
-        elif isinstance(data['l'], str) and len(data['l']) > 2 and \
-                data['l'].startswith('a:') and data['l'][2:] not in fields:
-            return INVALID
+        elif isinstance(data['l'], str) and len(data['l']) > 2:
+            prop = data['l']
+            if '(' in prop:
+                prop = self._remove_function_calls(prop)
+            if prop.startswith('a:') and prop[2:] not in fields:
+                return INVALID
 
         if isinstance(data['r'], dict):
             result = self._prune(fields, data['r'])
@@ -244,9 +286,12 @@ class BinaryOp:
                     data['r'] = {'l': 1, 'o': '==', 'r': 1}
                 else:
                     data['r'] = {'l': 1, 'o': '!=', 'r': 1}
-        elif isinstance(data['r'], str) and len(data['r']) > 2 and \
-                data['r'].startswith('a:') and data['r'][2:] not in fields:
-            return INVALID
+        elif isinstance(data['r'], str) and len(data['r']) > 2:
+            prop = data['r']
+            if '(' in prop:
+                prop = self._remove_function_calls(prop)
+            if prop.startswith('a:') and prop[2:] not in fields:
+                return INVALID
 
         if isinstance(data['r'], dict) and isinstance(data['l'], dict):
             if data['l']['l'] == 1 and data['l']['r'] == 1 and data['r']['l'] == 1 and data['r']['r'] == 1:
@@ -267,7 +312,15 @@ class BinaryOp:
         else:
             rhv, params, counter = self._process_op(self.rhv, params, counter)
 
-        return f'({lhv} {self.op.replace("==", "=")} {rhv})', params, counter
+        ret_op = self.op
+        if ret_op == 'startswith':
+            ret_op = 'like'
+            rhv = f"CONCAT({rhv}, '%')"
+        elif ret_op == 'endswith':
+            ret_op = 'like'
+            rhv = f"CONCAT('%', {rhv})"
+
+        return f'({lhv} {ret_op.replace("==", "=")} {rhv})', params, counter
 
     @staticmethod
     def _process_op(v, params: dict, counter: int):
